import { __rest } from "tslib";
import { deserializeDynamicMap } from '@cloudbase/cals';
import { isEmptyObj } from '../common';
import { generateDataBindForDataset } from './generate';
/**
 * 将对象的key的首字母转换为小写
 * { UserName: 'xxx', Id: 'xxxx'} => { userName: 'xxx', id: 'xxx' }
 * 默认只处理最外层, 若 deep 为 true, 则递归处理
 */
function lowercaseKey(obj, deep) {
    if (!obj || typeof obj !== 'object')
        return obj;
    if (Array.isArray(obj))
        return obj.map((item) => lowercaseKey(item, deep));
    const result = {};
    return Object.keys(obj).reduce((acc, key) => {
        const smallCase = key.charAt(0).toLowerCase() + key.slice(1);
        const val = obj[key];
        acc[smallCase] = deep ? lowercaseKey(val, deep) : val;
        return acc;
    }, result);
}
/**
 * 标准化从云API获取的数据源结构
 *  1. 将大写转换为小写
 *  2. 将 methods 和 methods 从字符串转换为对象
 */
function standardizeDataSource(ds) {
    const transformed = lowercaseKey(ds, false);
    try {
        // const { schema } = ds;
        if (typeof transformed.schema === 'string') {
            transformed.schema = JSON.parse(transformed.schema);
        }
        if (!transformed.schema || typeof transformed.schema !== 'object') {
            // console.warn(
            //   '[standardizeDataSource]invalid schema',
            //   schema,
            //   'in datasource',
            //   transformed
            // );
            transformed.schema = {};
        }
    }
    catch (error) {
        // console.warn(
        //   '[standardizeDataSource] unable to transform schema',
        //   transformed.schema
        // );
        transformed.schema = {};
    }
    try {
        const { methods } = ds;
        if (typeof transformed.methods === 'string') {
            transformed.methods = JSON.parse(transformed.methods);
        }
        if (!Array.isArray(transformed.methods)) {
            // console.warn(
            //   '[standardizeDataSource]invalid methods',
            //   methods,
            //   'in datasource',
            //   transformed
            // );
            transformed.methods = [];
        }
    }
    catch (error) {
        // console.warn(
        //   '[standardizeDataSource]unable to transform methods',
        //   transformed.methods
        // );
        transformed.methods = [];
    }
    return transformed;
}
/**
 * 简化数据源描述信息数组, 以供低码运行时使用
 * @param datasources 完整的数据源描述信息数组
 */
export function getDatasourceProfiles(datasources) {
    return ((datasources === null || datasources === void 0 ? void 0 : datasources.map((item) => {
        const ds = standardizeDataSource(item);
        return {
            id: ds.id,
            title: ds.title,
            name: ds.name,
            type: ds.type,
            childDataSourceNames: ds.childDataSourceNames,
            schema: ds.schema,
            methods: ds.methods.map((method) => {
                return {
                    name: method.name,
                    type: method.type,
                    outParams: method.outParams,
                };
            }),
        };
    })) || []);
}
function _generateDynamicDataset(ctx, dataset) {
    var _a;
    const { looseError = false } = ctx;
    // 这里的动作实际上是一个有副作用的动作，所以这里进行一个简单的深拷贝
    let result = JSON.parse(JSON.stringify(dataset));
    let { state } = result;
    if (state) {
        for (let key in state) {
            /**
             * 减少体积
             * 过滤运行态无用字段
             */
            let _b = state[key], { label: _, dataType: __ } = _b, config = __rest(_b, ["label", "dataType"]);
            if (config.varType === 'datasource' && ((_a = config.initMethod) === null || _a === void 0 ? void 0 : _a.params)) {
                let params = deserializeDynamicMap(ctx, config.initMethod.params || {});
                if (params && !isEmptyObj(params)) {
                    let processed = {};
                    for (let paramKey in params) {
                        let bind = params[paramKey];
                        try {
                            processed[paramKey] = generateDataBindForDataset({
                                propertyPath: paramKey,
                                bindDataPath: bind.value,
                                type: bind.type,
                            });
                        }
                        catch (e) {
                            if (!looseError) {
                                throw e;
                            }
                            else {
                                console.warn(`generate dataset ${key}'s param: ${paramKey} error:`, e);
                            }
                        }
                    }
                    config.initMethod.params = processed;
                }
            }
        }
    }
    return result;
}
function getDatasetProfiles(cals, apps, options) {
    const result = {};
    if (cals.dataset) {
        result.$global = _generateDynamicDataset({ app: cals, looseError: options.looseError }, cals.dataset);
    }
    (apps || []).forEach((app) => {
        var _a;
        (_a = app.items) === null || _a === void 0 ? void 0 : _a.forEach((page) => {
            var _a;
            if (page.dataset) {
                result[((_a = app.extra) === null || _a === void 0 ? void 0 : _a.rootPath) ? `${app.extra.rootPath}/${page.id}` : page.id] = _generateDynamicDataset({ app: cals, page, looseError: options.looseError }, page.dataset);
            }
        });
    });
    return result;
}
export function getConfig(calses, options) {
    var _a, _b, _c, _d, _e, _f, _g;
    if (calses && !Array.isArray(calses)) {
        calses = [calses];
    }
    const cals = calses[0];
    const BASE_CONFIG = window._WEDA_RUNTIME_CONFIG || {};
    let isPrivate = false;
    let tcbApiOrigin = '';
    let privatelink = (_a = cals === null || cals === void 0 ? void 0 : cals.extra) === null || _a === void 0 ? void 0 : _a.privatelinkConfig;
    if (Object.prototype.hasOwnProperty.call(BASE_CONFIG, 'isPrivate')) {
        isPrivate = BASE_CONFIG.isPrivate;
    }
    else {
        isPrivate = window._isPrivate || false;
    }
    if (Object.prototype.hasOwnProperty.call(BASE_CONFIG, 'tcbApiOrigin')) {
        tcbApiOrigin = BASE_CONFIG.tcbApiOrigin;
    }
    else if (isPrivate && ((_b = cals.extra) === null || _b === void 0 ? void 0 : _b.domain)) {
        tcbApiOrigin = `https://${(_c = cals.extra) === null || _c === void 0 ? void 0 : _c.domain}`;
    }
    /**
     * 实时预览需要根据应用绑定（自定义应用或企业工作台）两种开启立即生效
     * 引起无法全局开启，需要是否开启判断
     * 全局变量会在企业工作台开启时开启，但是自定义应用要独立开启
     */
    // if (Object.prototype.hasOwnProperty.call(BASE_CONFIG, 'privatelink')) {
    //   privatelink = BASE_CONFIG.privatelink;
    // } else {
    //   privatelink = (window as any)._PRIVATE_LINK_META;
    // }
    const envID = BASE_CONFIG.envId || ((_d = cals.extra) === null || _d === void 0 ? void 0 : _d.envId) || cals.envId;
    return {
        /** 当前是否处于正式发布模式 */
        isProd: false || ((_e = options === null || options === void 0 ? void 0 : options.deployOptions) === null || _e === void 0 ? void 0 : _e.mode) === 'production',
        /** 低码应用ID */
        appID: cals.id || cals.appId || '',
        /** 云开发环境ID */
        envID,
        /** 应用端ID */
        tcbClientId: ((_f = cals.extra) === null || _f === void 0 ? void 0 : _f.enableLoginStatusShare) ? envID : (_g = cals.extra) === null || _g === void 0 ? void 0 : _g.clientId,
        /** 数据源描述对象数组 */
        dataSourceProfiles: getDatasourceProfiles(cals.dataSources || []),
        /**
         * 新的dataset变量配置对象
         * key 为页面ID(全局为$global), val 为变量配置数组
         */
        datasetProfiles: getDatasetProfiles(cals, calses, {
            looseError: options === null || options === void 0 ? void 0 : options.looseError,
        }),
        tcbApiOrigin: tcbApiOrigin || '',
        isPrivate: isPrivate || false,
        privatelink: privatelink
            ? Object.assign({ jsUrl: {
                    vm: 'https://qbase.cdn-go.cn/lcap/lcap-resource-cdngo/-/release/_npm/@cloudbase/privatelink@0.0.1/dist/cdn/cloudbase.privatelink.vm.js',
                    vender: `https://qbase.cdn-go.cn/lcap/lcap-resource-cdngo/-/release/_npm/@cloudbase/privatelink-vender@0.0.2/dist/cdn/cloudbase.privatelink.vender.js`,
                } }, privatelink) : undefined,
    };
}
